package kr.co.vividnext.sodalive.audio_content

import kr.co.vividnext.sodalive.audio_content.detail.PutAudioContentLikeRequest
import kr.co.vividnext.sodalive.audio_content.donation.AudioContentDonationRequest
import kr.co.vividnext.sodalive.audio_content.order.OrderRequest
import kr.co.vividnext.sodalive.audio_content.order.OrderType
import kr.co.vividnext.sodalive.user.CreatorFollowRequestRequest
import kr.co.vividnext.sodalive.user.UserApi
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.util.TimeZone

class AudioContentRepository(
    private val api: AudioContentApi,
    private val userApi: UserApi
) {
    fun getAudioContentListByCurationId(
        curationId: Long,
        page: Int,
        size: Int,
        sort: AudioContentViewModel.Sort = AudioContentViewModel.Sort.NEWEST,
        token: String
    ) = api.getAudioContentListByCurationId(
        id = curationId,
        page = page - 1,
        size = size,
        sort = sort,
        authHeader = token
    )

    fun getAudioContentList(
        id: Long,
        page: Int,
        size: Int,
        sort: AudioContentViewModel.Sort,
        token: String
    ) = api.getAudioContentList(
        id = id,
        page = page - 1,
        size = size,
        sort = sort,
        authHeader = token
    )

    fun getAudioContentThemeList(token: String) = api.getAudioContentThemeList(token)

    fun uploadAudioContent(
        coverImage: MultipartBody.Part,
        contentFile: MultipartBody.Part,
        request: RequestBody,
        token: String
    ) = api.uploadAudioContent(
        coverImage = coverImage,
        contentFile = contentFile,
        request = request,
        authHeader = token
    )

    fun modifyAudioContent(
        coverImage: MultipartBody.Part? = null,
        request: RequestBody,
        token: String
    ) = api.modifyAudioContent(
        coverImage = coverImage,
        request = request,
        authHeader = token
    )

    fun deleteAudioContent(
        id: Long,
        token: String
    ) = api.deleteAudioContent(
        id = id,
        authHeader = token
    )

    fun getAudioContentDetail(audioContentId: Long, token: String) = api.getAudioContentDetail(
        id = audioContentId,
        timezone = TimeZone.getDefault().id,
        authHeader = token
    )

    fun registerNotification(
        creatorId: Long,
        token: String
    ) = userApi.creatorFollow(
        request = CreatorFollowRequestRequest(creatorId = creatorId),
        authHeader = token
    )

    fun unRegisterNotification(
        creatorId: Long,
        token: String
    ) = userApi.creatorUnFollow(
        request = CreatorFollowRequestRequest(creatorId = creatorId),
        authHeader = token
    )

    fun orderContent(
        contentId: Long,
        orderType: OrderType,
        token: String
    ) = api.orderAudioContent(
        request = OrderRequest(
            contentId = contentId,
            orderType = orderType,
            container = "aos"
        ),
        authHeader = token
    )

    fun getAudioContentOrderList(
        page: Int,
        size: Int,
        token: String
    ) = api.getAudioContentOrderList(
        page = page - 1,
        size = size,
        authHeader = token
    )

    fun addAllPlaybackTracking(
        request: AddAllPlaybackTrackingRequest,
        token: String
    ) = api.addAllPlaybackTracking(request, authHeader = token)

    fun likeContent(
        request: PutAudioContentLikeRequest,
        token: String
    ) = api.likeContent(request, authHeader = token)

    fun getMain(token: String) = api.getMain(authHeader = token)

    fun getNewContentOfTheme(theme: String, token: String) = api.getNewContentOfTheme(
        theme = theme,
        authHeader = token
    )

    fun getNewContentAllOfTheme(
        theme: String,
        page: Int,
        size: Int,
        token: String
    ) = api.getNewContentAllOfTheme(
        theme = theme,
        page = page - 1,
        size = size,
        authHeader = token
    )

    fun getNewContentThemeList(token: String) = api.getNewContentThemeList(authHeader = token)

    fun donation(
        contentId: Long,
        can: Int,
        comment: String,
        token: String
    ) = api.donation(
        request = AudioContentDonationRequest(
            contentId = contentId,
            donationCan = can,
            comment = comment
        ),
        authHeader = token
    )
}
